//
//  NPSideMenuObject.m
//  NPSideMenu
//
//  Created by Nicolai Persson on 06/02/13.
//  Copyright (c) 2013 Nicolai Persson. All rights reserved.
//

#import "NPSideMenuObject.h"
#import <QuartzCore/QuartzCore.h>

#define kAnimationDuration 0.2f

@interface NPSideMenu (Private)
- (void)setFramesAccordingToOrientation;
@end

@implementation NPSideMenu (Private)
- (void)setFramesAccordingToOrientation {
    CGRect bounds = UIScreen.mainScreen.bounds, rect;
    CGFloat statusBarHeight;
    
    UIInterfaceOrientation orientation = UIApplication.sharedApplication.statusBarOrientation;

    if (UIInterfaceOrientationIsPortrait(orientation)) {
        statusBarHeight = (UIApplication.sharedApplication.statusBarHidden) ? 0.0f : UIApplication.sharedApplication.statusBarFrame.size.height;
        
        if (self.leftMenu) {
            rect = bounds;
            rect.origin       = CGPointZero;
            rect.origin.x     = (orientation == UIInterfaceOrientationPortrait) ? 0.0f : bounds.size.width - self.leftMenu.width;
            rect.size.width   = self.leftMenu.width;
            rect.origin.y     = statusBarHeight;
            rect.size.height -= statusBarHeight;
            
            self.leftMenu.view.frame = rect;
            self.leftMenu.tableView.frame = rect;
        }
        
        if (self.rightMenu) {
            rect = bounds;
            rect.origin       = CGPointZero;
            rect.origin.x     = (orientation != UIInterfaceOrientationPortrait) ? 0.0f : bounds.size.width - self.rightMenu.width;
            rect.size.width   = self.rightMenu.width;
            rect.origin.y     = statusBarHeight;
            rect.size.height -= statusBarHeight;
            
            self.rightMenu.view.frame = rect;
            self.rightMenu.tableView.frame = rect;
        }
    } else {
        statusBarHeight = (UIApplication.sharedApplication.statusBarHidden) ? 0.0f : UIApplication.sharedApplication.statusBarFrame.size.width;
        
        if (self.leftMenu) {
            rect = bounds;
            rect.origin       = CGPointZero;
            rect.origin.y     = (orientation != UIInterfaceOrientationLandscapeLeft) ? 0.0f : bounds.size.height - self.leftMenu.width;
            rect.size.height  = self.leftMenu.width;
            rect.size.width  -= statusBarHeight;
            
            if (orientation == UIInterfaceOrientationLandscapeLeft)
                rect.origin.x     = statusBarHeight;
            
            self.leftMenu.view.frame = rect;
            self.leftMenu.tableView.frame = rect;
        }
        
        if (self.rightMenu) {
            rect = bounds;
            rect.origin       = CGPointZero;
            rect.origin.y     = (orientation == UIInterfaceOrientationLandscapeLeft) ? 0.0f : bounds.size.height - self.rightMenu.width;
            rect.size.height  = self.rightMenu.width;
            rect.size.width  -= statusBarHeight;
            
            if (orientation == UIInterfaceOrientationLandscapeLeft)
                rect.origin.x     = statusBarHeight;
            
            self.rightMenu.view.frame = rect;
            self.rightMenu.tableView.frame = rect;
        }
    }
}
@end

@implementation NPSideMenu
@synthesize canDrag, controller, useNavigationBarOnly, leftMenu, rightMenu;

/*********************************************************************************
 ***  Static
 *********************************************************************************/

+ (NPSideMenu *)sharedInstance {
    static dispatch_once_t once;
    static NPSideMenu *instance;
    
    dispatch_once(&once, ^{
        instance = [[NPSideMenu alloc] init];
    });
    
    return instance;
}

/*********************************************************************************
 ***  Init
 *********************************************************************************/

- (id)init {
    if (self = [super init]) {
        canTap = NO;
        
        panGestureRecognizer = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(controllerPanned:)];
        tapGestureRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(controllerTapped:)];
        
        panGestureRecognizer.maximumNumberOfTouches = 1;
        panGestureRecognizer.minimumNumberOfTouches = 1;
        panGestureRecognizer.delegate               = self;
        
        tapGestureRecognizer.delegate = self;
        
        useNavigationBarOnly = YES;
        
        canDrag = YES;
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(didRotate:)
                                                     name:UIApplicationDidChangeStatusBarOrientationNotification
                                                   object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(playerOpened)
                                                     name:@"kDRRadioPlayerOpened"
                                                   object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(playerClosed)
                                                     name:@"kDRRadioPlayerClosed"
                                                   object:nil];
    }
    
    return self;
}

- (void)dealloc {
    [NSNotificationCenter.defaultCenter removeObserver:self
                                                  name:@"UIDeviceOrientationDidChangeNotification"
                                                object:nil];
}

- (void)configureNavigationController:(UINavigationController *)navigationController {
    if (!navigationController.view.superview) {
        @throw [NSException exceptionWithName:NSInvalidArgumentException
                                       reason:@"The view controller is expected to have a superview. This one doesn't."
                                     userInfo:nil];
    }
    
    self.controller = navigationController;
}

- (void)didRotate:(NSNotification *)notification {
    CGFloat angle = 0.0f;
    
    switch (UIApplication.sharedApplication.statusBarOrientation) {
        case UIInterfaceOrientationPortraitUpsideDown:
            angle = M_PI;
            break;
        case UIInterfaceOrientationLandscapeLeft:
            angle = -M_PI / 2.0f;
            break;
        case UIInterfaceOrientationLandscapeRight:
            angle = M_PI / 2.0f;
            break;
        default:
            angle = 0.0f;
            break;
    }
    
    self.leftMenu.view.transform  = CGAffineTransformMakeRotation(angle);
    self.rightMenu.view.transform = CGAffineTransformMakeRotation(angle);
    
    self.leftMenu.hidden  = YES;
    self.rightMenu.hidden = YES;
    self.leftMenu.view.hidden  = YES;
    self.rightMenu.view.hidden = YES;
    canTap = NO;
    animating = NO;
    
    [self setFramesAccordingToOrientation];
}


/*********************************************************************************
 ***  Setters
 *********************************************************************************/

- (void)setController:(UINavigationController *)value {
    if (controller) {
        [panGestureRecognizer.view removeGestureRecognizer:panGestureRecognizer];
        [tapGestureRecognizer.view removeGestureRecognizer:tapGestureRecognizer];
    }
    
    controller = value;
    
    self.leftMenu = leftMenu;
    self.rightMenu = rightMenu;
    self.useNavigationBarOnly = useNavigationBarOnly;
}

- (void)setControllerOffset:(CGFloat)offset {
    CGRect frame = self.controller.view.frame;
    
    switch (self.controller.interfaceOrientation) {
        case UIInterfaceOrientationPortrait:
            frame.origin.x = offset;
            break;
            
        case UIInterfaceOrientationPortraitUpsideDown:
            frame.origin.x = -1*offset;
            break;
            
        case UIInterfaceOrientationLandscapeRight:
            frame.origin.y = offset;
            break;
            
        case UIInterfaceOrientationLandscapeLeft:
            frame.origin.y = -1*offset;
            break;
            
        default:
            break;
    }
    
    self.controller.view.frame = frame;
}

- (void)setLeftMenu:(NPSideMenuController *)value {
    if (!value) return;
    
    if (value.width < 1.0f) {
        @throw [NSException exceptionWithName:NSInvalidArgumentException
                                       reason:@"You forgot to set the width of the left menu. Try harder."
                                     userInfo:nil];
    }
    
    leftMenu = value;
    leftMenu.hidden = YES;
    leftMenu.view.hidden = YES;
    
    [controller.view.superview insertSubview:self.leftMenu.view belowSubview:controller.view];
    [self setFramesAccordingToOrientation];
}

- (void)setRightMenu:(NPSideMenuController *)value {
    if (!value) return;
    
    if (value.width < 1.0f) {
        @throw [NSException exceptionWithName:NSInvalidArgumentException
                                       reason:@"You forgot to set the width of the right menu. Try harder."
                                     userInfo:nil];
    }
    
    rightMenu = value;
    rightMenu.hidden = YES;
    rightMenu.view.hidden = YES;
    
    [controller.view.superview insertSubview:self.rightMenu.view belowSubview:controller.view];
    [self setFramesAccordingToOrientation];
}

- (void)setUseNavigationBarOnly:(BOOL)value {
    useNavigationBarOnly = value;
    
    [panGestureRecognizer.view removeGestureRecognizer:panGestureRecognizer];
    [tapGestureRecognizer.view removeGestureRecognizer:tapGestureRecognizer];
    
    if (useNavigationBarOnly) {
        gestureView = self.controller.navigationBar;
        [self.controller.navigationBar addGestureRecognizer:panGestureRecognizer];
    } else {
        gestureView = self.controller.view;
        [self.controller.view addGestureRecognizer:panGestureRecognizer];
    }
    
    [self.controller.view addGestureRecognizer:tapGestureRecognizer];
}

/*********************************************************************************
 ***  Frame + point getters
 *********************************************************************************/

- (CGRect)controllerFrameAdjustedTointerfaceOrientation {
    if (UIInterfaceOrientationIsPortrait(self.controller.interfaceOrientation)) {
        return self.controller.view.frame;
    } else {
        return CGRectMake(self.controller.view.frame.origin.y, self.controller.view.frame.origin.x, self.controller.view.frame.size.height, self.controller.view.frame.size.width);
    }
}

- (CGPoint)controllerPointAdjustedTointerfaceOrientation {
    CGPoint result = self.controller.view.frame.origin;
    
    if (UIInterfaceOrientationIsPortrait(self.controller.interfaceOrientation)) {
        result.x = ABS(result.x);
        result.y = ABS(result.y);
    } else {
        result.x = ABS(result.y);
        result.y = ABS(result.x);
    }
    
    return result;
}

/*********************************************************************************
 ***  Toggles
 *********************************************************************************/

- (void)toggleLeftMenu {
    if (!self.leftMenu)
        return;
    
    if (self.leftMenu.hidden) {
        self.leftMenu.view.hidden = NO;
        
        [NSNotificationCenter.defaultCenter postNotificationName:kNPSideMenuShowLeft object:nil];
        [UIView animateWithDuration:kAnimationDuration animations:^{
            [self setControllerOffset:self.leftMenu.width];
        } completion:^(BOOL finished) {
            if (!finished)
                return;
            
            self.controller.visibleViewController.view.userInteractionEnabled = NO;
            self.leftMenu.hidden = NO;
            canTap = YES;
        }];
    } else {
        [self hideAllMenus];
    }
}

- (void)showLeftMenu {
    if (!self.leftMenu || !self.leftMenu.hidden)
        return;
    
    [self toggleRightMenu];
}

- (void)toggleRightMenu {
    if (!self.rightMenu)
        return;
    
    if (self.rightMenu.hidden) {
        self.rightMenu.view.hidden = NO;
        
        [NSNotificationCenter.defaultCenter postNotificationName:kNPSideMenuShowRight object:nil];
        [UIView animateWithDuration:kAnimationDuration animations:^{
            [self setControllerOffset:-1*self.rightMenu.width];
        } completion:^(BOOL finished) {
            if (!finished)
                return;
            
            self.controller.visibleViewController.view.userInteractionEnabled = NO;
            self.rightMenu.hidden = NO;
            canTap = YES;
        }];
    } else {
        [self hideAllMenus];
    }
}

- (void)showRightMenu {
    if (!self.rightMenu || !self.rightMenu.hidden)
        return;
    
    [self toggleRightMenu];
}

- (void)hideAllMenus {
    if (animating)
        return;
    
    [NSNotificationCenter.defaultCenter postNotificationName:kNPSideMenuHideLeft object:nil];
    [NSNotificationCenter.defaultCenter postNotificationName:kNPSideMenuHideRight object:nil];
    
    canTap = NO;
    
    animating = YES;
    self.controller.visibleViewController.view.userInteractionEnabled = YES;
    [UIView animateWithDuration:kAnimationDuration animations:^{
        [self setControllerOffset:0.0f];
    } completion:^(BOOL finished) {
        self.leftMenu.hidden  = YES;
        self.rightMenu.hidden = YES;
        self.leftMenu.view.hidden  = YES;
        self.rightMenu.view.hidden = YES;
        animating = NO;
    }];
}

/*********************************************************************************
 ***  Gesture handlers
 *********************************************************************************/

- (void)controllerPanned:(UIPanGestureRecognizer *)gestureRecognizer {
    if (animating)
        return;
    
    NSInteger what = 0;
    CGFloat x, n;
    
    if (gestureRecognizer.state == UIGestureRecognizerStateBegan) {
        origin = self.controllerPointAdjustedTointerfaceOrientation;
	}
    
    CGPoint velocity  = [gestureRecognizer velocityInView:self.controller.view],
    translatedPoint = [gestureRecognizer translationInView:self.controller.view];
    
    if (leftMenu && (!rightMenu || rightMenu.hidden) && ((translatedPoint.x > 0 && leftMenu.hidden) || (translatedPoint.x < 0 && !leftMenu.hidden))) {
        what = 1;
        
        translatedPoint   = CGPointMake(origin.x+translatedPoint.x, origin.y+translatedPoint.y);
        translatedPoint.x = MIN(translatedPoint.x, self.leftMenu.width);
        translatedPoint.x = MAX(translatedPoint.x, 0);
        
        n = self.leftMenu.width / 2.0f;
        
        self.leftMenu.view.hidden  = NO;
        self.rightMenu.view.hidden = YES;
        
        [self setControllerOffset:translatedPoint.x];
    } else if (rightMenu && (!self.leftMenu || self.leftMenu.hidden) && ((translatedPoint.x < 0 && rightMenu.hidden) || (translatedPoint.x > 0 && !rightMenu.hidden))) {
        what = 2;
        
        translatedPoint   = CGPointMake(translatedPoint.x - origin.x, translatedPoint.y - origin.y);
        translatedPoint.x = MIN(translatedPoint.x, 0.0f);
        translatedPoint.x = MAX(translatedPoint.x, -self.rightMenu.width);
        
        n = self.rightMenu.width / 2.0f;
        
        self.leftMenu.view.hidden  = YES;
        self.rightMenu.view.hidden = NO;
        
        [self setControllerOffset:translatedPoint.x];
    }
    
	if (gestureRecognizer.state == UIGestureRecognizerStateEnded) {
        x = translatedPoint.x + 0.15f * velocity.x;
        
        switch (what) {
            case 1: {
                if (x > n) {
                    animating = YES;
                    [UIView animateWithDuration:kAnimationDuration animations:^{
                        [self setControllerOffset:n*2.0f];
                    } completion:^(BOOL finished) {
                        [NSNotificationCenter.defaultCenter postNotificationName:kNPSideMenuShowLeft object:nil];
                        self.controller.visibleViewController.view.userInteractionEnabled = NO;
                        self.leftMenu.hidden = NO;
                        canTap = YES;
                        animating = NO;
                    }];
                } else {
                    [self hideAllMenus];
                }
                
                break;
            }
                
            case 2: {
                if (x < -1*n) {
                    animating = YES;
                    [UIView animateWithDuration:kAnimationDuration animations:^{
                        [self setControllerOffset:-1*(n*2.0f)];
                    } completion:^(BOOL finished) {
                        [NSNotificationCenter.defaultCenter postNotificationName:kNPSideMenuShowRight object:nil];
                        self.controller.visibleViewController.view.userInteractionEnabled = NO;
                        self.rightMenu.hidden = NO;
                        canTap = YES;
                        animating = NO;
                    }];
                } else {
                    [self hideAllMenus];
                }
                
                break;
            }
                
            default:
                break;
        }
    }
}

- (void)controllerTapped:(UITapGestureRecognizer *)gestureRecognizer {
    [self hideAllMenus];
}

/*********************************************************************************
 ***  UIGestureRecognizerDelegate
 *********************************************************************************/

- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceiveTouch:(UITouch *)touch {
    if ([gestureRecognizer isKindOfClass:[UIPanGestureRecognizer class]])
        return ![touch.view isKindOfClass:[UISlider class]];
    
    return self.canDrag;
}

- (BOOL)gestureRecognizerShouldBegin:(UIGestureRecognizer *)gestureRecognizer {
    if ([gestureRecognizer isKindOfClass:[UITapGestureRecognizer class]] && !canTap) {
        return NO;
    }
    
    return self.canDrag;
}

- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldRecognizeSimultaneouslyWithGestureRecognizer:(UIGestureRecognizer *)otherGestureRecognizer {
	return ![gestureRecognizer isEqual:panGestureRecognizer];
}

@end
